# !/usr/bin/env python
# -*- coding:utf-8 -*-

import argparse

import logging as log


LOG_LEVELS = {'critical': log.CRITICAL,
              'error': log.ERROR,
              'warn': log.WARNING,
              'warning': log.WARNING,
              'info': log.INFO,
              'debug': log.DEBUG,
              }


def log_and_raise(message):
    """ Raises a critical error and logs with given `error_message` """
    log.critical(message)
    raise Exception(message)


def set_up_logger(level, file_name):
    """Sets up logger which always writes to the console and if provided also to `file_name`"""
    log_formatter = log.Formatter("%(asctime)s — %(levelname)s — %(funcName)s:%(lineno)d — %(message)s")
    root_logger = log.getLogger()
    root_logger.setLevel(level)

    if file_name:
        file_handler = log.FileHandler(file_name, mode="w")
        file_handler.setFormatter(log_formatter)
        root_logger.addHandler(file_handler)

    console_handler = log.StreamHandler()
    console_handler.setFormatter(log_formatter)
    root_logger.addHandler(console_handler)


def arg_handling_make_config(defaults):
    """Handles command line arguments and returns `input_file` and `run_config` for make_config routine"""
    parser = argparse.ArgumentParser()
    parser.add_argument("-f",
                        "--file",
                        required=True,
                        help=("Provide path to configuration file"
                              "Example: -f <path/to/configuration/file.yaml>'"),
                        )

    add_loglevel_argument(parser, defaults["log_level"], LOG_LEVELS)
    add_output_argument(parser, defaults["output_file"])
    add_logfile_argument(parser)

    args = parser.parse_args()

    input_file = args.file
    level = LOG_LEVELS.get(args.log.lower())
    output_file = args.output
    log_file = args.logfile

    run_config = {"log_level": level,
                  "output_file": output_file,
                  "log_file": log_file,
                  }

    return input_file, run_config


def arg_handling_convert_results(defaults):
    """Handles command line arguments and returns `input_file` and `run_config` for convert_results routine"""
    parser = argparse.ArgumentParser()
    parser.add_argument("-f",
                        "--file",
                        required=True,
                        help=("Provide path to protobuf file"
                              "Example: -f <path/to/result/file.pb>'"),
                        )

    add_loglevel_argument(parser, defaults["log_level"], LOG_LEVELS)
    add_logfile_argument(parser)
    add_select_agents_argument(parser)

    args = parser.parse_args()

    input_file = args.file
    level = LOG_LEVELS.get(args.log.lower())
    log_file = args.logfile
    agents_to_extract = args.agents

    run_config = {"log_level": level,
                  "log_file": log_file,
                  "agents_to_extract": agents_to_extract,
                  }

    return input_file, run_config


def add_select_agents_argument(parser):
    """Adds argument handling for selecting agent types which get converted"""
    parser.add_argument("-a",
                        "--agents",
                        nargs="*",
                        type=str,
                        help=("Provide list of agents to extract. "
                              "Example --agents MyAgent1 MyAgent2 default='None'"),
                        )


def add_logfile_argument(parser):
    """Adds argument handling for setting the logfile"""
    parser.add_argument("-lf",
                        "--logfile",
                        help=("Provide logging file. "
                              "Example --logfile <path/to/log/file.log>', default='None'"),
                        )


def add_output_argument(parser, defaults):
    """Adds argument handling for setting the outputfile"""
    parser.add_argument("-o",
                        "--output",
                        default=defaults,
                        help=("Provide path to config.pb file. "
                              "Example --output <path/to/config.pb>', default='config.pb'"),
                        )


def add_loglevel_argument(parser, defaults, levels):
    """Adds argument handling for setting the log_level"""
    parser.add_argument("-l",
                        "--log",
                        default=defaults,
                        choices=list(levels.keys()),
                        help=("Provide logging level. "
                              "Example --log debug', default='info'"),
                        )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: AgentMessage.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='AgentMessage.proto',
  package='communication',
  syntax='proto2',
  serialized_options=b'\n\024de.dlr.fame.protobufB\005Agent',
  serialized_pb=b'\n\x12\x41gentMessage.proto\x12\rcommunication\"\x99\x01\n\rProtoDataItem\x12\x12\n\ndataTypeId\x18\x01 \x02(\x05\x12\x11\n\tboolValue\x18\x02 \x03(\x08\x12\x10\n\x08intValue\x18\x03 \x03(\x05\x12\x11\n\tlongValue\x18\x04 \x03(\x03\x12\x12\n\nfloatValue\x18\x05 \x03(\x02\x12\x13\n\x0b\x64oubleValue\x18\x06 \x03(\x01\x12\x13\n\x0bstringValue\x18\x07 \x03(\t\"\xda\x01\n\nNestedItem\x12\x12\n\ndataTypeId\x18\x01 \x02(\x05\x12\x11\n\tboolValue\x18\x02 \x03(\x08\x12\x10\n\x08intValue\x18\x03 \x03(\x05\x12\x11\n\tlongValue\x18\x04 \x03(\x03\x12\x12\n\nfloatValue\x18\x05 \x03(\x02\x12\x13\n\x0b\x64oubleValue\x18\x06 \x03(\x01\x12\x13\n\x0bstringValue\x18\x07 \x03(\t\x12\x14\n\x0ctimeSeriesId\x18\x08 \x03(\x05\x12,\n\tcomponent\x18\t \x03(\x0b\x32\x19.communication.NestedItem\"\x93\x01\n\x0cProtoMessage\x12\x10\n\x08senderId\x18\x01 \x02(\x03\x12\x12\n\nreceiverId\x18\x02 \x02(\x03\x12.\n\x08\x64\x61taItem\x18\x03 \x03(\x0b\x32\x1c.communication.ProtoDataItem\x12-\n\nnestedItem\x18\x04 \x03(\x0b\x32\x19.communication.NestedItemB\x1d\n\x14\x64\x65.dlr.fame.protobufB\x05\x41gent'
)




_PROTODATAITEM = _descriptor.Descriptor(
  name='ProtoDataItem',
  full_name='communication.ProtoDataItem',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='dataTypeId', full_name='communication.ProtoDataItem.dataTypeId', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='boolValue', full_name='communication.ProtoDataItem.boolValue', index=1,
      number=2, type=8, cpp_type=7, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='intValue', full_name='communication.ProtoDataItem.intValue', index=2,
      number=3, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='longValue', full_name='communication.ProtoDataItem.longValue', index=3,
      number=4, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='floatValue', full_name='communication.ProtoDataItem.floatValue', index=4,
      number=5, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='doubleValue', full_name='communication.ProtoDataItem.doubleValue', index=5,
      number=6, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stringValue', full_name='communication.ProtoDataItem.stringValue', index=6,
      number=7, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=38,
  serialized_end=191,
)


_NESTEDITEM = _descriptor.Descriptor(
  name='NestedItem',
  full_name='communication.NestedItem',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='dataTypeId', full_name='communication.NestedItem.dataTypeId', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='boolValue', full_name='communication.NestedItem.boolValue', index=1,
      number=2, type=8, cpp_type=7, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='intValue', full_name='communication.NestedItem.intValue', index=2,
      number=3, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='longValue', full_name='communication.NestedItem.longValue', index=3,
      number=4, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='floatValue', full_name='communication.NestedItem.floatValue', index=4,
      number=5, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='doubleValue', full_name='communication.NestedItem.doubleValue', index=5,
      number=6, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stringValue', full_name='communication.NestedItem.stringValue', index=6,
      number=7, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timeSeriesId', full_name='communication.NestedItem.timeSeriesId', index=7,
      number=8, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='component', full_name='communication.NestedItem.component', index=8,
      number=9, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=194,
  serialized_end=412,
)


_PROTOMESSAGE = _descriptor.Descriptor(
  name='ProtoMessage',
  full_name='communication.ProtoMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='senderId', full_name='communication.ProtoMessage.senderId', index=0,
      number=1, type=3, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='receiverId', full_name='communication.ProtoMessage.receiverId', index=1,
      number=2, type=3, cpp_type=2, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dataItem', full_name='communication.ProtoMessage.dataItem', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nestedItem', full_name='communication.ProtoMessage.nestedItem', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=415,
  serialized_end=562,
)

_NESTEDITEM.fields_by_name['component'].message_type = _NESTEDITEM
_PROTOMESSAGE.fields_by_name['dataItem'].message_type = _PROTODATAITEM
_PROTOMESSAGE.fields_by_name['nestedItem'].message_type = _NESTEDITEM
DESCRIPTOR.message_types_by_name['ProtoDataItem'] = _PROTODATAITEM
DESCRIPTOR.message_types_by_name['NestedItem'] = _NESTEDITEM
DESCRIPTOR.message_types_by_name['ProtoMessage'] = _PROTOMESSAGE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ProtoDataItem = _reflection.GeneratedProtocolMessageType('ProtoDataItem', (_message.Message,), {
  'DESCRIPTOR' : _PROTODATAITEM,
  '__module__' : 'AgentMessage_pb2'
  # @@protoc_insertion_point(class_scope:communication.ProtoDataItem)
  })
_sym_db.RegisterMessage(ProtoDataItem)

NestedItem = _reflection.GeneratedProtocolMessageType('NestedItem', (_message.Message,), {
  'DESCRIPTOR' : _NESTEDITEM,
  '__module__' : 'AgentMessage_pb2'
  # @@protoc_insertion_point(class_scope:communication.NestedItem)
  })
_sym_db.RegisterMessage(NestedItem)

ProtoMessage = _reflection.GeneratedProtocolMessageType('ProtoMessage', (_message.Message,), {
  'DESCRIPTOR' : _PROTOMESSAGE,
  '__module__' : 'AgentMessage_pb2'
  # @@protoc_insertion_point(class_scope:communication.ProtoMessage)
  })
_sym_db.RegisterMessage(ProtoMessage)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Bundle.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import AgentMessage_pb2 as AgentMessage__pb2
import Services_pb2 as Services__pb2
import InputFile_pb2 as InputFile__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='Bundle.proto',
  package='mpi',
  syntax='proto2',
  serialized_options=b'\n\024de.dlr.fame.protobufB\003Mpi',
  serialized_pb=b'\n\x0c\x42undle.proto\x12\x03mpi\x1a\x12\x41gentMessage.proto\x1a\x0eServices.proto\x1a\x0fInputFile.proto\"\xc3\x02\n\nMpiMessage\x12\x33\n\rscheduledTime\x18\x01 \x01(\x0b\x32\x1c.communication.ScheduledTime\x12,\n\x06warmUp\x18\x02 \x01(\x0b\x32\x1c.communication.WarmUpMessage\x12/\n\x0b\x61\x64\x64ressBook\x18\x03 \x01(\x0b\x32\x1a.communication.AddressBook\x12\"\n\x05input\x18\x04 \x01(\x0b\x32\x13.iobinary.InputData\x12%\n\x06output\x18\x05 \x01(\x0b\x32\x15.communication.Output\x12,\n\x07message\x18\x06 \x01(\x0b\x32\x1b.communication.ProtoMessage\x12(\n\x05setup\x18\x07 \x01(\x0b\x32\x19.communication.ProtoSetup\"*\n\x06\x42undle\x12 \n\x07message\x18\x01 \x03(\x0b\x32\x0f.mpi.MpiMessageB\x1b\n\x14\x64\x65.dlr.fame.protobufB\x03Mpi'
  ,
  dependencies=[AgentMessage__pb2.DESCRIPTOR,Services__pb2.DESCRIPTOR,InputFile__pb2.DESCRIPTOR,])




_MPIMESSAGE = _descriptor.Descriptor(
  name='MpiMessage',
  full_name='mpi.MpiMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='scheduledTime', full_name='mpi.MpiMessage.scheduledTime', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='warmUp', full_name='mpi.MpiMessage.warmUp', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='addressBook', full_name='mpi.MpiMessage.addressBook', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='input', full_name='mpi.MpiMessage.input', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='output', full_name='mpi.MpiMessage.output', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='message', full_name='mpi.MpiMessage.message', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='setup', full_name='mpi.MpiMessage.setup', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=75,
  serialized_end=398,
)


_BUNDLE = _descriptor.Descriptor(
  name='Bundle',
  full_name='mpi.Bundle',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='message', full_name='mpi.Bundle.message', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=400,
  serialized_end=442,
)

_MPIMESSAGE.fields_by_name['scheduledTime'].message_type = Services__pb2._SCHEDULEDTIME
_MPIMESSAGE.fields_by_name['warmUp'].message_type = Services__pb2._WARMUPMESSAGE
_MPIMESSAGE.fields_by_name['addressBook'].message_type = Services__pb2._ADDRESSBOOK
_MPIMESSAGE.fields_by_name['input'].message_type = InputFile__pb2._INPUTDATA
_MPIMESSAGE.fields_by_name['output'].message_type = Services__pb2._OUTPUT
_MPIMESSAGE.fields_by_name['message'].message_type = AgentMessage__pb2._PROTOMESSAGE
_MPIMESSAGE.fields_by_name['setup'].message_type = Services__pb2._PROTOSETUP
_BUNDLE.fields_by_name['message'].message_type = _MPIMESSAGE
DESCRIPTOR.message_types_by_name['MpiMessage'] = _MPIMESSAGE
DESCRIPTOR.message_types_by_name['Bundle'] = _BUNDLE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

MpiMessage = _reflection.GeneratedProtocolMessageType('MpiMessage', (_message.Message,), {
  'DESCRIPTOR' : _MPIMESSAGE,
  '__module__' : 'Bundle_pb2'
  # @@protoc_insertion_point(class_scope:mpi.MpiMessage)
  })
_sym_db.RegisterMessage(MpiMessage)

Bundle = _reflection.GeneratedProtocolMessageType('Bundle', (_message.Message,), {
  'DESCRIPTOR' : _BUNDLE,
  '__module__' : 'Bundle_pb2'
  # @@protoc_insertion_point(class_scope:mpi.Bundle)
  })
_sym_db.RegisterMessage(Bundle)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# !/usr/bin/env python
# -*- coding:utf-8 -*-

import pathlib as pt
import logging as log

from fameio.source.FieldValidator import FieldValidator, FieldType
from fameio.source.TimeSeriesManager import TimeSeriesManager
from fameio.source.tools import arg_handling_make_config, log_and_raise, set_up_logger
from fameio.source.loader import load_yaml
from fameio.source.FameTime import FameTime
from fameio.protobuf_definitions import InputFile_pb2


DEFAULT_CONFIG = {"log_level": "warning",
                  "output_file": "config.pb",
                  "log_file": None,
                  }


def set_general_properties(properties, proto):
    """Set the general simulation properties in the given proto_buffer"""
    for property_name, property_value in properties.items():
        if not hasattr(property_value, "keys"):
            setattr(proto, property_name, property_value)
        else:
            parent = getattr(proto, property_name)
            for child_property_name, child_property_value in property_value.items():
                if child_property_name == "StartTime" or child_property_name == "StopTime":
                    child_property_value = int(FameTime.convert_time_string_to_fame_time_step(child_property_value))
                setattr(parent, child_property_name, child_property_value)
        log.info("Set general properties for `{}`".format(property_name))


def update_field_values(pb_agent, fields, time_series_manager, field_validator):
    """Adds all fields in the given list to the given agent proto buffer"""
    if fields is not None:
        agent_type = pb_agent.className
        for field_name, field_value in fields.items():
            pb_field = pb_agent.field.add()
            pb_field.fieldName = field_name
            field_type = field_validator.get_field_type(agent_type, field_name)
            if field_validator.is_valid(agent_type, field_name, field_value):
                if field_type is FieldType.INTEGER:
                    pb_field.intValue = field_value
                elif field_type is FieldType.DOUBLE:
                    pb_field.doubleValue.extend([field_value])
                elif field_type is FieldType.ENUM:
                    pb_field.stringValue = field_value
                elif field_type is FieldType.TIME_SERIES:
                    if isinstance(field_value, str):
                        file_name = pt.Path(field_value).as_posix()
                        pb_field.seriesId = time_series_manager.save_get_time_series_id(file_name)
                    else:
                        pb_field.seriesId = time_series_manager.save_get_time_series_id(field_value)
                elif field_type is FieldType.DOUBLE_LIST:
                    for element in field_value:
                        pb_field.doubleValue.extend([element])
                else:
                    log_and_raise("FieldType '{}' not implemented.".format(field_type))
            else:
                log_and_raise("'{}' not allowed in field '{}' of agent {}".format(field_value, field_name, pb_agent.id))


def get_or_error(agent, param):
    """Gets given `param` from dictionary or raises error if field is missing"""
    try:
        return agent[param]
    except KeyError:
        log_and_raise("Cannot find '{}' in `agent` {}".format(param, agent))


def set_agents_and_time_series(agent_list, proto_buffer, field_validator):
    """
    Iterates through all agents, adds them and all of their fields to the given proto buffer and also
    adds all referenced files as time series to the proto_buffer. Ensures proper field parameterization and format.
    """
    time_series_manager = TimeSeriesManager()
    for agent in agent_list:
        agent = convert_keys_to_lower(agent)
        pb_agent = proto_buffer.Agent.add()
        pb_agent.className = get_or_error(agent, "type")
        pb_agent.id = get_or_error(agent, "id")
        if "attributes" in agent:
            fields = agent.get("attributes")
            update_field_values(pb_agent, fields, time_series_manager, field_validator)
        log.info("Set `Attributes` for agent `{}` with ID `{}`".format(pb_agent.className, pb_agent.id))
    time_series_manager.add_time_series_to_proto_buffer(proto_buffer)


def convert_keys_to_lower(agent):
    """Returns given dictionary with `keys` in lower case"""
    return {keys.lower(): value for keys, value in agent.items()}


def set_contracts(contracts, proto_buffer):
    """Adds all contracts in the given list to the given proto buffer"""
    for contract in contracts:
        pb_contract = proto_buffer.Contract.add()
        for key, value in contract.items():
            setattr(pb_contract, key, value)
    log.info("Added contracts to protobuf file.")


def write_protobuf_to_disk(output_file, proto_input_data):
    """Writes given `protobuf_input_data` in `output_file` to disk"""
    f = open(output_file, "wb")
    f.write(proto_input_data.SerializeToString())
    f.close()
    log.info("Saved protobuf file `{}` to disk".format(output_file))


def run(file, config=DEFAULT_CONFIG):
    """Executes the main workflow for the building of a FAME configuration file"""
    set_up_logger(level=config["log_level"], file_name=config["log_file"])

    config_data = load_yaml(file)
    validator = FieldValidator(config_data["Schema"])
    proto_input_data = InputFile_pb2.InputData()
    set_general_properties(config_data["GeneralProperties"], proto_input_data)
    set_agents_and_time_series(config_data["Agents"], proto_input_data, validator)

    contract_list = config_data["Contracts"]
    set_contracts(contract_list, proto_input_data)

    write_protobuf_to_disk(config["output_file"], proto_input_data)

    log.info("Completed conversion of all input in `{}` to protobuf file `{}`".format(file, config["output_file"]))


if __name__ == '__main__':
    input_file, run_config = arg_handling_make_config(DEFAULT_CONFIG)
    run(input_file, run_config)
